" Vim filetype plugin
" Language: nginx.conf
" Maintainer: Chris Aumann <me@chr4.org>
" Last Change: Apr 15, 2017
"              2023 Aug 28 by Vim Project (undo_ftplugin)

setlocal commentstring=#\ %s

let b:undo_ftplugin = "setlocal commentstring<"
